@@ -17,12 +17,13 @@ import re
 import traceback
 from collections import namedtuple
 from docutils import nodes
+from sphinx import addnodes
 from sphinx.util.osutil import ensuredir
 
 import blockdiag.utils.rst.nodes
 import blockdiag.utils.rst.directives
 from blockdiag.utils.bootstrap import detectfont
-from blockdiag.utils.compat import u
+from blockdiag.utils.compat import u, string_types
 from blockdiag.utils.fontmap import FontMap
 
 # fontconfig; it will be initialized on `builder-inited` event.
@@ -37,8 +38,10 @@ class blockdiag_node(blockdiag.utils.rst.nodes.blockdiag):
             filename = self.get_abspath(image_format, builder)
 
         antialias = builder.config.blockdiag_antialias
+        transparency = builder.config.blockdiag_transparency
         image = super(blockdiag_node, self).to_drawer(image_format, filename, fontmap,
-                                                      antialias=antialias, **kwargs)
+                                                      antialias=antialias, transparency=transparency,
+                                                      **kwargs)
         for node in image.diagram.traverse_nodes():
             node.href = resolve_reference(builder, node.href)
 
@@ -58,7 +61,9 @@ class blockdiag_node(blockdiag.utils.rst.nodes.blockdiag):
                        fontmap=builder.config.blockdiag_fontmap,
                        format=image_format)
 
-        if hasattr(builder, 'imgpath'):
+        if hasattr(builder, 'imagedir'):  # Sphinx (>= 1.3.x)
+            outputdir = os.path.join(builder.outdir, builder.imagedir)
+        elif hasattr(builder, 'imgpath'):  # Sphinx (<= 1.2.x) and HTML writer
             outputdir = os.path.join(builder.outdir, '_images')
         else:
             outputdir = builder.outdir
@@ -75,24 +80,28 @@ class Blockdiag(blockdiag.utils.rst.directives.BlockdiagDirective):
         return node
 
 
-def get_anchor(builder, refid):
-    for docname in builder.env.found_docs:
-        doctree = builder.env.get_doctree(docname)
-        for target in doctree.traverse(nodes.Targetable):
-            if target.attributes.get('refid') == refid:
-                targetfile = builder.get_relative_uri(builder.current_docname, docname)
-                return targetfile + "#" + refid
-
-
 def resolve_reference(builder, href):
     if href is None:
-        return
+        return None
+
     pattern = re.compile(u("^:ref:`(.+?)`"), re.UNICODE)
     matched = pattern.search(href)
-    if matched:
-        return get_anchor(builder, matched.group(1))
-    else:
+    if matched is None:
         return href
+    else:
+        refid = matched.group(1)
+        domain = builder.env.domains['std']
+        node = addnodes.pending_xref(refexplicit=False)
+        xref = domain.resolve_xref(builder.env, builder.current_docname, builder,
+                                   'ref', refid, node, node)
+        if xref:
+            if 'refid' in xref:
+                return "#" + xref['refid']
+            else:
+                return xref['refuri']
+        else:
+            builder.warn('undefined label: %s' % refid)
+            return None
 
 
 def html_render_svg(self, node):
@@ -246,7 +255,7 @@ def on_builder_inited(self):
 
     try:
         fontpath = self.builder.config.blockdiag_fontpath
-        if isinstance(fontpath, blockdiag.utils.compat.string_types):
+        if isinstance(fontpath, string_types):
             fontpath = [fontpath]
 
         if fontpath:
@@ -298,6 +307,7 @@ def setup(app):
     app.add_config_value('blockdiag_fontpath', None, 'html')
     app.add_config_value('blockdiag_fontmap', None, 'html')
     app.add_config_value('blockdiag_antialias', False, 'html')
+    app.add_config_value('blockdiag_transparency', True, 'html')
     app.add_config_value('blockdiag_debug', False, 'html')
     app.add_config_value('blockdiag_html_image_format', 'PNG', 'html')
     app.add_config_value('blockdiag_tex_image_format', None, 'html')  # backward compatibility for 1.3.1
